/*
  This Energia code example exercises four HDC3020 devices (Digital %RH and Temperature sensor).
  The human interface to this firmware is done via UART, using a terminal program such as Teraterm, SerialPlot or Hyperterminal.

  Datasheets:
  -------------------------------------------------
  HDC3020:    https://www.ti.com/lit/ds/symlink/hdc3020.pdf

  User Guides:
  -------------------------------------------------
  MSP-EXP430F5529LP: https://www.ti.com/lit/ug/slau533d/slau533d.pdf

  Product Pages:
  -------------------------------------------------
  HDC3020:          https://www.ti.com/product/HDC3020

  This code example exercises four HDC3020 devices sequentially at 100kHz over I2C bus addresses listed below.

  I2C Addresses:
  ---------------------------------------------------
  I2C Addresses (8 bit ==> 7 bit ==> decimal (for Energia/Ardunio))
    -------------
                 8 bit    7 bit
  HDC3020 #1 ==> 0x88 ==> 0x44
  HDC3020 #2 ==> 0x8A ==> 0x45
  HDC3020 #3 ==> 0x8C ==> 0x46
  HDC3020 #4 ==> 0x8E ==> 0x47
    -------------
  ADVANCED FEATURE USAGE:
  N/A

  HARDWARE CONNECTIONS / PIN USAGE:
  -------------------------------------------------
                               3.3V 3.3V
                               /|\  /|\
              HDC3020 %RH/T    4.7k 4.7k    MSP430F5529
              I2C responder     |    |    MCU I2C controller
            -----------------   |    |   -------------------
           |    ADDR 0x44    |
           |V+            SDA|<-|----+->|P4.1/UCB1SDA       |
         |-|ADDR0         SCL|<-+------>|P4.2/UCB1SCL   P1.0|--> LED (RED on LP)
         |-|ADDR1            |          |        )      P4.7|--> LED2 (GREEN on LP)
         |-|GND              |          |                   |
         |  -----------------            -------------------
         -
              HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x45    |
         |-|V+            SDA|<-|----+
         |-|ADDR0         SCL|<-+-----
           |                 |
         |-|ADDR1            |
         |-|GND              |
         |  -----------------
         -

                HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x46    |
         |-|V+            SDA|<-|----+
         |-|ADDR1         SCL|<-+-----
           |                 |
         |-|ADDR0            |
         |-|GND              |
         |  -----------------
         -

      HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x47    |
         |-|V+            SDA|<-|----+
         |-|ADDR0         SCL|<-+-----
         |-|ADDR1            |
           |                 |
          |-|GND             |
          |  ----------------
          -

  UART OUTPUT (@115200, 8N1)
  ------------
  This code puts out comma separated calculated results on UART of the LaunchPad for logging purposes (using Teraterm or similar) into .csv file, for use in MSExcel -
  for example:

  ,HDC302x 4 I2C Address Example
  ,Enter '1' to begin reading HDC3020 devices %RH and Temp
  ,HDC3 ADDR44 %RH ,  HDC3 ADDR44 TEMP(C), HDC3 ADDR45 %RH,  HDC3 ADDR45 TEMP(C),HDC3 ADDR46 %RH ,  HDC3 ADDR46 TEMP(C), HDC3 ADDR47 %RH,  HDC3 ADDR47 TEMP(C)
  ,    49.7047     ,        22.4338      ,    53.8277     ,        22.4525      ,    49.0227     ,        22.4178      ,    49.8650     ,        22.5673
  ,    49.6818     ,        22.4552      ,    53.8079     ,        22.4525      ,    48.9967     ,        22.4178      ,    49.8314     ,        22.5673
  ,    49.6712     ,        22.4338      ,    53.8048     ,        22.4739      ,    48.9799     ,        22.4018      ,    49.7963     ,        22.5673
  ,    49.6910     ,        22.4765      ,    53.8018     ,        22.4525      ,    48.9662     ,        22.4018      ,    49.7917     ,        22.5887
  ,    49.7078     ,        22.4552      ,    53.7957     ,        22.4525      ,    48.9631     ,        22.4178      ,    49.7719     ,        22.5460

  JOSH WYATT
  TEXAS INSTRUMENTS INC.
  12500 TI BLVD.
  DALLAS, TEXAS 75243
  TEMPERATURE AND HUMIDITY SENSOR PRODUCTS
  APPLICATIONS TEAM
  © 2023
*/
#include <Wire.h>
#define LED RED_LED //P1.0 on MSP430F5529 LP
#define LED2 GREEN_LED //P4.7 on MSP430F5529 LP (heartbeat)

// constants that won't change.
const int ledPin =  RED_LED;      // the number of the LED pin
const int ledPin2 =  GREEN_LED;   // the number of the LED pin

// HDC3x variables that will change:
float HDC3x_hum_percentage;
float HDC3x_temp_celcius;
uint16_t HDC3x_hum_MSB;
uint16_t HDC3x_hum_DEC;
uint16_t HDC3x_temp_MSB;
uint16_t HDC3x_temp_DEC;
uint16_t HDC3x_temp;
uint8_t  iBuff_HDC3x[4];

void setup() {
  // initialize serial communication:
  Serial.begin(115200);

  // initialize the LED pins:
  pinMode(ledPin, OUTPUT);
  pinMode(ledPin2, OUTPUT);
  digitalWrite(LED, LOW);   // turn the LED off by making the voltage LOW
  digitalWrite(LED2, LOW);

  // join i2c bus
  Wire.begin();
  Wire.setTimeout(0);
  delay(100);
  Serial.println("");
  Serial.print(",");
  Serial.println("HDC302x 4 I2C Address Example");
  Serial.print(",");
  Serial.println("Enter '1' to begin reading HDC3020 devices %RH and Temp");
}

void loop() {
  // use the sensor:
  if (Serial.available() > 0) {
    int inByte = Serial.read();
    switch (inByte) {
      case '1':
        Serial.print(",");
        Serial.println("HDC3 ADDR44 %RH ,  HDC3 ADDR44 TEMP(C), HDC3 ADDR45 %RH,  HDC3 ADDR45 TEMP(C),HDC3 ADDR46 %RH ,  HDC3 ADDR46 TEMP(C), HDC3 ADDR47 %RH,  HDC3 ADDR47 TEMP(C)"); // Print board data logfile column headers
        while ((Serial.peek() == -1))
        {
          digitalWrite(ledPin2, HIGH);
          digitalWrite(ledPin, LOW);

          //HDC3 @ address 0x44//
          Wire.beginTransmission(0x44);                                                                 // transmit to HDC3x device address 0x44
          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
          Wire.endTransmission();                                                                       // stop transmitting
          delay(100);
          Wire.requestFrom(0x44, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
          // math here
          while (Wire.available())
          {
            Wire.readBytes(iBuff_HDC3x, 6);
            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                      // shift left
            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                          // do math according to the HDC3x datasheet
            Serial.print(",    ");
            HDC3x_hum_percentage = HDC3x_hum_percentage;
            Serial.print(HDC3x_hum_percentage, 4);                                                    // print out %RH to terminal
            Serial.print("     ,        ");
            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                   // shift left
            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                       // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                      // do math according to the HDC3x datasheet
            Serial.print(HDC3x_temp_celcius, 4);
          }

          //HDC3 @ address 0x45//
          Wire.beginTransmission(0x45);                                                                 // transmit to HDC3x device address 0x45
          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
          Wire.endTransmission();                                                                       // stop transmitting
          delay(25);
          Wire.requestFrom(0x45, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
          // math here
          while (Wire.available())
          {
            Wire.readBytes(iBuff_HDC3x, 6);
            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                      // shift left
            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                          // do math according to the HDC3x datasheet
            Serial.print("      ,    ");
            HDC3x_hum_percentage = HDC3x_hum_percentage;
            Serial.print(HDC3x_hum_percentage, 4);                                                    // print out %RH to terminal
            Serial.print("     ,        ");
            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                   // shift left
            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                       // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                      // do math according to the HDC3x datasheet
            Serial.print(HDC3x_temp_celcius, 4);
          }

          //HDC3 @ address 0x46//
          Wire.beginTransmission(0x46);                                                                 // transmit to HDC3x device address 0x46
          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
          Wire.endTransmission();                                                                       // stop transmitting
          delay(25);
          Wire.requestFrom(0x46, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
          // math here
          while (Wire.available())
          {
            Wire.readBytes(iBuff_HDC3x, 6);
            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                      // shift left
            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                          // do math according to the HDC3x datasheet
            Serial.print("      ,    ");
            HDC3x_hum_percentage = HDC3x_hum_percentage;
            Serial.print(HDC3x_hum_percentage, 4);                                                    // print out %RH to terminal
            Serial.print("     ,        ");
            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                   // shift left
            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                       // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                      // do math according to the HDC3x datasheet
            Serial.print(HDC3x_temp_celcius, 4);
          }

          //HDC3 @ address 0x47//
          Wire.beginTransmission(0x47);                                                                 // transmit to HDC3x device address 0x47
          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
          Wire.endTransmission();                                                                       // stop transmitting
          delay(25);
          Wire.requestFrom(0x47, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
          // math here
          while (Wire.available())
          {
            Wire.readBytes(iBuff_HDC3x, 6);
            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                      // shift left
            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                          // do math according to the HDC3x datasheet
            Serial.print("      ,    ");
            HDC3x_hum_percentage = HDC3x_hum_percentage;
            Serial.print(HDC3x_hum_percentage, 4);                                                    // print out %RH to terminal
            Serial.print("     ,        ");
            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                   // shift left
            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                       // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                      // do math according to the HDC3x datasheet
            Serial.println(HDC3x_temp_celcius, 4);
          }
          digitalWrite(ledPin, HIGH);
          digitalWrite(ledPin2, LOW);
          delay(1000);
        }
        break;

      default:
        // turn all the LEDs off:
        digitalWrite(ledPin2, LOW);
        digitalWrite(ledPin, LOW);
    }
  }
}
